/****************************************************************************
 * Grafik-LCD Treiber mit VRAM.
 *
 * (C) Andreas Kaiser 2008.
 *
 * Die Software darf frei kopiert und verndert werden, solange sie
 * nicht ohne meine ausdrckliche Erlaubnis fr kommerzielle Zwecke
 * eingesetzt wird.
 ****************************************************************************/

#include "config.h"
#include "vram.h"
#include "lcd.h"
#include "graph.h"

#define LCD_WORDS	(LCD_COLS/LCD_BITS)

#if LCD_BITS == 8
  uint8_t	g_bitmask[8] = { 0b10000000,0b01000000,0b00100000,0b00010000,
				 0b00001000,0b00000100,0b00000010,0b00000001 };
  uint8_t	g_setmask[8] = { 0b11111111,0b01111111,0b00111111,0b00011111,
				 0b00001111,0b00000111,0b00000011,0b00000001 };
#else
  uint8_t	g_bitmask[4] = { 0b10001000,0b01000100,0b00100010,0b00010001 };
  uint8_t	g_setmask[4] = { 0b11111111,0b01110111,0b00110011,0b00010001 };
#endif

//-----------------------------------------------------------------------------

uint8_t
g_read(vram_row_t row, vram_col_t col)
{
    uint8_t data;
    row += lcd_home;
    if (LCD_DOUBLE) {
	if (row < LCD_ROWS/2)
	    data = vram_read_i(row, col) & 0xF0;
	else
	    data = vram_read_i(row - LCD_ROWS/2, col) << 4;
    } else if (LCD_BITS == 4) {
	data = vram_read_i(row, col) & 0xF0;
    } else
	data = vram_read_i(row, col);
    return data;
}

void
g_write(vram_row_t row, vram_col_t col, g_op op, uint8_t mask, uint8_t data)
{
    //PRINTF("g_write %d,%d,%d,%02X,%02X\n", row, col, op, mask, data);
    row += lcd_home;
    if (LCD_DOUBLE) {
	if (row < LCD_ROWS/2) {
	    mask &= 0xF0;
	} else {
	    mask >>= 4;
	    row -= LCD_ROWS/2;
	}
    } else if (LCD_BITS == 4)
	mask &= 0xF0;
    switch (op) {
    case G_Set:
	data = 0xFF;
	break;
    case G_Clear:
    	data = 0x00;
	break;
    case G_Invert:
	data = vram_read_i(row, col) ^ mask;
	break;
    case G_Replace:
	break;
    }
    vram_write_i(row, col, mask, data);
}

//-----------------------------------------------------------------------------

void
g_clear(void)
{
    lcd_set_home(0);
    vram_row_t row = 0;
    g_gpos(0, 0);
    g_cpos(0, 0);
    do {
	vram_col_t col = 0;
	do {
	    //vram_write_i(row, col, 0xFF, (((row >> 2) ^ col) & 1) ? 0xFF: 0x00);
	    vram_write_i(row, col, 0xFF, 0x00);
	} while (col++ != LCD_WORDS-1);
    } while (row++ != LCD_ROWS-1);
}
