/****************************************************************************
 * Grafik-LCD Treiber mit VRAM.
 *
 * (C) Andreas Kaiser 2008.
 *
 * Die Software darf frei kopiert und verndert werden, solange sie
 * nicht ohne meine ausdrckliche Erlaubnis fr kommerzielle Zwecke
 * eingesetzt wird.
 ****************************************************************************/

#include "config.h"
#include "lcd.h"
#include "vram.h"
#include "graph.h"
#include "usart.h"

int
uart_putchar(char c, FILE *stream)
{
    usartTransmit(c);
    return 0;
}

FILE uart_stdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

//-----------------------------------------------------------------------------

static void
init(void)
{
    // pullup unused pins
    setBits(PORTB, 1<<PB0);
    setBits(PORTC, 1<<PC1 | 1<<PC0);
    setBits(PORTD, 1<<PC4 | 1<<PC3 | 1<<PC1 | 1<<PC0);
#ifdef PORTE
    setBits(PORTE, 1<<PE0 | 1<<PE1 | 1<<PE2);
#endif

    usartInit();
    stdout = &uart_stdout;

    vram_init();
    lcd_init(); // After vram_init, because of SCLK control.

    sei();
}

//-----------------------------------------------------------------------------

static void
start(void)
{
    usartStart();
    PRINTF("\n" __DATE__ ", " __TIME__ "\n");

    g_clear();
    if (!g_hardfont(8, 12))
    //if (!g_hardfont(8, 10))
    //if (!g_hardfont(6, 8))
    //if (!g_hardfont(32, 48))
	PRINTF("hardfont failed\n");

    _delay_ms(100);
    lcd_enable();

}

//-----------------------------------------------------------------------------

int
main(void)
{
    init();
    start();

#if 1
    g_gpos(20, 120); g_gtext_P("    0 Degrees", G_Replace, 0);
    g_gpos(20, 120); g_gtext_P("    45 Degrees", G_Set, 45);
    g_gpos(20, 120); g_gtext_P("    90 Degrees", G_Set, 90);
    g_gpos(20, 120); g_gtext_P("    -45 Degrees", G_Set, -45);
    g_gpos(20, 120); g_gtext_P("    -90 Degrees", G_Set, -90);
    g_cpos(0, 0); g_text_P(__DATE__ "\n" __TIME__ "\n");
    g_line(0, 0, 319, 239, G_Invert);
    g_fill(50, 50, 150, 150, G_Invert);
    g_fill(100, 100, 200, 200, G_Invert);
    g_fill(100, 100, 200, 200, G_Invert);
#else
    //g_cpos(0, 0); g_text_P("Hello\nWorld\n");
    g_cpos(0, 0); g_text_P(__DATE__ " " __TIME__ "\n");
#endif
    for (int first = 1;;) {
	unsigned c = usartReceive();
	if (first) {
	    first = 0;
	    g_clear();
	}
  	g_char(c);
    }
}
